#
# Kamailio configuration
#     created by Geoff Mina (gmina@connectfirst.com)
#     modified by Daniel-Constantin Mierla (miconda@gmail.com) 
#
# Please refer to reference http://www.kamailio.org/dokuwiki/doku.php
# for a full description of all modules and functions
#
#


###### Global Parameters #####
debug=1
log_stderror=no
log_facility=LOG_LOCAL0
fork=yes
children=16
disable_tcp=yes
listen=eth0:5060
listen=eth1:5060
server_header="Server: G-Tel SIP Gateway"

#fork=no
#log_stderror=yes

##### Module Loading and Param Setting #####
mpath="/usr/local/lib64/kamailio/modules/"
loadmodule "sl.so"
loadmodule "db_mysql.so"
loadmodule "maxfwd.so"
loadmodule "uri.so"
loadmodule "textops.so"

## Enable FIFO ##
loadmodule "mi_fifo.so"
modparam("mi_fifo", "fifo_name", "/tmp/kamailio_fifo")

## Enable TM ##
loadmodule "tm.so"
modparam("tm","fr_timer",15)
modparam("tm","fr_inv_timer",30)

## Enable RR ##
loadmodule "rr.so"
modparam("rr", "enable_full_lr",0)
modparam("rr", "append_fromtag",1)
modparam("rr", "enable_double_rr",1)
modparam("rr", "add_username",0)

## Enable Logging ##
loadmodule "xlog.so"
modparam("xlog", "buf_size",4096)
modparam("xlog", "force_color",0)

## Enable SipTrace module for debugging SIP transactions ##
loadmodule "siptrace.so"
modparam("siptrace","db_url","mysql://appservers:appservers@localhost/opensips")
modparam("siptrace","table","sip_trace")
modparam("siptrace","trace_on",0)
modparam("siptrace","trace_flag",13)

## Enable Dispatcher module ##
loadmodule "dispatcher.so"
modparam("dispatcher","flags",2)
modparam("dispatcher","dst_avp","$avp(i:271)")
modparam("dispatcher","grp_avp","$avp(i:272)")
modparam("dispatcher","cnt_avp","$avp(i:273)")
modparam("dispatcher","ds_ping_method","OPTIONS")
modparam("dispatcher","ds_ping_from","sip:monitoring@connectfirst.com")
modparam("dispatcher","ds_ping_interval",30)
modparam("dispatcher","ds_probing_mode",1)
modparam("dispatcher","ds_probing_threshhold", 2)

###########################################################################
## Request route 'main'
###########################################################################
route{
	
	## 
	## Start with some simple logging and sip tracing
	## For the time being we are going to sip trace everything
	## just to make sure all packets are flowing properly
	##
	xlog("L_INFO", "New request - M=$rm RURI=$ru F=$fu T=$tu IP=$si ID=$ci\n");
	setflag(13);
	sip_trace();
	
	
	##
	## Initial sanity check to ensure the message isn't too big
	##
	if(msg:len > 8192){
		xlog("L_INFO", "Message too big\n");
		t_reply("513", "Message Too Big");
		exit;
	}

	##
	## Ensure we aren't in a tight loop for some reason
	## this number could probably be lower.
	##
	if (!mf_process_maxfwd_header("70")){
		xlog("L_INFO", "Too many hops\n");
		t_reply("483", "Too Many Hops");
		exit;
	}

	##
	## Filter out SIP messages that should not come though this system
	##
	if(is_method("REGISTER|MESSAGE")){
		xlog("L_INFO", "This SIP method is not handled here [$rm]\n");
		sl_send_reply("404", "Not here");
		exit;
	}
	
	if(is_method("OPTIONS")){
		##
		## If the method is an OPTIONS we are simply going to respond
		## with a 200 OK.  The internal monitoring platform (nagios) will be
		## monitoring the health of the gateway via the check_sip plugin
		##
		xlog("L_INFO", "Method is an OPTIONS, probably just monitoring\n");
		sl_send_reply("200","OK");
		exit;
	}

	if(has_totag()) {
		##
		## If we are in-dialog loose_route() should return true and we should
		## end up inside next IF.
		##	
		if (loose_route()) {
			xlog("L_INFO", "Loose route has returned true, attempting routing.\n");
			route(2);
			exit;
		}

		if ( is_method("ACK") ) {
			# relay only ACKs belonging to an active transaction
			# - otherwise we have no routing info for it (should have
			# gone via above loose_route() IF)
			if ( t_check_trans() ) {
				t_relay();
			}
			exit;
		}
		# we do all the time Record Route
		# within-dialog messages without Route are rejected
		sl_send_reply("404","Not here");
		exit;
	}

	# remove preloaded Route headers, if any
	# some phones do add it when have an Outbound proxy set (Sofia SIP stack)
	remove_hf("Route");

	# drop any ACK here
	# within dialog ACK should have been gone via has_totag() + loose_route()
	if(is_method("ACK"))
		exit;
	##
	## CANCEL processing - just relay, TM module takes care
	##
	if(is_method("CANCEL")){
		xlog("L_INFO", "We have a CANCEL");
		if (t_check_trans())
			t_relay();
		exit;
	}

	# we should get here only initial INVITE	
	if(!is_method("INVITE")){
		# everything comming here is replied - no route for it
		sl_send_reply("404", "Not here");
		exit;
	}

	record_route();

	##
	## If the method is an INVITE we are going to enter the
	## route[1] method which is responsible for invoking the 
	## dispatcher and relaying the request to the end point
	##
	xlog("L_INFO", "Method is an INVITE, fetching next from dispatcher\n");
	route(1);
	exit;
}



########################################################################
## Handles relay of INVITE messages
## with round-robin load balancing
##
## We select the next from the dispatcher
## using simple round-robin algorithm.
########################################################################
route[1]{
	# attempt to forward only when we have a destination
	if(ds_select_domain("1","4"))
	{
		t_on_reply("1");
		t_on_failure("1");
		if(!t_relay()){
			sl_reply_error();
		}
		exit;
	}
	sl_send_reply("500", "No destination");
	exit;
}



########################################################################
## Handles relay of all non INVITE messages 
## 
## All messages which were routed via the loose_route
## condition will end up here.  If there is a message that fails
## to t_rely there probably isn't much we can do other than
## return an error.
########################################################################
route[2]{
	xlog("L_INFO", "Setting up reply handler and relaying request\n");
	t_on_reply("1");
	if(!t_relay()){
		sl_reply_error();
	}
}



#######################################################################
## Simply logs responses
#######################################################################
onreply_route[1]{
	xlog("L_INFO", "Reply - S=$rs D=$rr F=$fu T=$tu IP=$si ID=$ci\n");
	exit;
}



#######################################################################
## Handles failure of INVITE forwarding
#######################################################################
failure_route[1]{
	xlog("L_INFO","Failure route, trying again, code [$T_reply_code]\n");

	# check first for cancelled transactions
	# we get here because the invite will have 487 reply code
	if(t_was_cancelled()){
		xlog("L_INFO","Transaction was cancelled\n");
		exit;
	}

	# next destination only if this proxy timeouts
	# time-out from downstream means destination was ok, ringing
	# but no answer
	if(t_local_replied("all") && t_check_status("408")){
		xlog("L_INFO","Got a 408 Timeout, flagging dest as invalid\n");
		ds_mark_dst();
		# relay again if we have new destination
		if(ds_next_dst())
		{
			t_on_reply("1");
			t_on_failure("1");
			t_relay();
		}
		# let the reply go upstream - it is default action
		exit;
	}
	
	# check for 500, 503, 504 - compact regexp
	if(t_check_status("500|503|504")){
		xlog("L_INFO","Got a 500 Series error, let's try the next destination, but not flag as invalid.\n");
		# relay again if we have new destination
		if(ds_next_dst())
		{
			t_on_reply("1");
			t_on_failure("1");
			t_relay();
		}
		# let the reply go upstream - it is default action
		exit;
	}
}
